﻿using System.Collections.Generic;
using System.Linq;
using Interfaces;
using StructureMap;

namespace Hr
{
    class _Employee
    {
        public string EmpNo { get; set; }
        public string ChineseName { get; set; }
        public string Descriptions { get; set; }
    }

    public class HrManager : IHr
    {
        public HrManager()
        {
            ICommon log = ObjectFactory.GetInstance<ICommon>();
            log.Log(LogType.Information, "Hr", "HrManager inited.");
        }

        private static readonly List<_Employee> _data = new List<_Employee>
        {
            new _Employee{EmpNo = "0001", ChineseName = "蒋国纲", Descriptions = "技术宅男"},
            new _Employee{EmpNo = "0002", ChineseName = "周星驰", Descriptions = "喜剧之王"},
            new _Employee{EmpNo = "0003", ChineseName = "李逍遥",Descriptions = "蜀山派掌门人"}
        };

        public IEnumerable<Employee> GetAllEmployees()
        {
            return from e in _data select new Employee {EmpNo = e.EmpNo, ChineseName = e.ChineseName};
        }

        public EmployeeDetail GetEmployeeByNo(string strEmpNo)
        {
            _Employee emp = _data.FirstOrDefault(e => e.EmpNo == strEmpNo);
            if (emp!=null)
            {
                IOffice office = ObjectFactory.GetInstance<IOffice>();
                return new EmployeeDetail
                       {
                            EmpNo = strEmpNo,
                            ChineseName = emp.ChineseName,
                            Descriptions = emp.Descriptions,
                            TaskAmount = office.GetTaskAmountOfEmployee(strEmpNo)
                       };
            }
            return null;
        }

        public int GetEmployeeAmount()
        {
            return _data.Count;
        }

        public void AddEmployee(Employee emp)
        {
            _data.Add(new _Employee{
                EmpNo = emp.EmpNo,
                ChineseName = emp.ChineseName
            });
        }

        public string GetEmpNameByNo(string strEmpNo)
        {
//             IOffice office = ObjectFactory.GetInstance<IOffice>();
//             office.Foo();

            _Employee emp = _data.FirstOrDefault(e => e.EmpNo == strEmpNo);
            if (emp!=null)
            {
                return emp.ChineseName;
            }
            return string.Empty;
        }
    }
}
