﻿using System;
using System.Collections.Generic;
using System.Linq;
using Interfaces;
using StructureMap;

namespace Office
{
    class _Task
    {
        public int Id { get; set; }
        public string EmpNo { get; set; }
        public string Descriptions { get; set; }
    }

    public class OfficeManager:IOffice
    {
        public OfficeManager()
        {
            ICommon log = ObjectFactory.GetInstance<ICommon>();
            log.Log(LogType.Information, "Office", "OfficeManager inited.");
        }

        private static readonly List<_Task> _data = new List<_Task>
                                               {
                                                   new _Task{Id = 1, EmpNo = "0001", Descriptions = "干点什么呢？"},
                                                   new _Task{Id = 2, EmpNo = "0002", Descriptions = "演戏了！"},
                                                   new _Task{Id = 3, EmpNo = "0001", Descriptions = "写代码啦。"},
                                                   new _Task{Id = 4, EmpNo = "0003", Descriptions = "打怪。"}
                                               };

        public IEnumerable<Task> GetAllTasks()
        {
            return _data.Select(t => new Task{Id = t.Id, EmpNo = t.EmpNo, Descriptions = t.Descriptions});
        }

        public TaskDetail GetTask(int id)
        {
            _Task task = _data.FirstOrDefault(t => t.Id == id);
            if (task!=null)
            {
                IHr hr = ObjectFactory.GetInstance<IHr>();
                return new TaskDetail
                {
                    Id = task.Id,
                    EmpNo = task.EmpNo,
                    Descriptions = task.Descriptions,
                    EmpName = hr.GetEmpNameByNo(task.EmpNo)
                };
            }
            return null;
        }

        public void AddTask(string strEmpNo, string strDesc)
        {
            _data.Add(new _Task
                {
                    Id = _data.Select(t => t.Id).Max(),
                    EmpNo = strEmpNo,
                    Descriptions = strDesc
                });
        }

        public void DeleteTask(int id)
        {
            _Task task = _data.FirstOrDefault(t => t.Id == id);
            if(task!=null)
            {
                _data.Remove(task);
            }
        }

        public int GetTaskAmountOfEmployee(string strEmpNo)
        {
            return _data.Count(t => t.EmpNo==strEmpNo);
        }

        public void Foo()
        {
            Console.WriteLine("This is OfficeManager::Foo");
        }
    }
}
